@@ -18,7 +18,7 @@ use Carp qw(croak);
 @ISA     = qw(Business::OnlinePayment::HTTPS);
 $me      = 'Business::OnlinePayment::Litle';
 $DEBUG   = 0;
-$VERSION = '0.901';
+$VERSION = '0.910';
 
 =head1 NAME
 
@@ -26,7 +26,7 @@ Business::OnlinePayment::Litle - Litle & Co. Backend for Business::OnlinePayment
 
 =head1 VERSION
 
-Version 0.900
+Version 0.910
 
 =cut
 
@@ -313,6 +313,19 @@ sub map_request {
         $amount = sprintf( "%.2f", $content->{amount} );
         $amount =~ s/\.//g;
     }
+    
+    #  put in a list of constraints
+    my @validate = (
+      [ 'city', 35 ],
+      [ 'address', 35 ],
+      [ 'state', 30 ],
+      [ 'name', 100 ],
+    );
+    foreach my $trunc ( @validate ) {
+      if( defined $content->{ $trunc->[0] } ) {
+        $content->{ $trunc->[0] } = substr($content->{ $trunc->[0] } , 0, $trunc->[1] );
+      }
+    };
 
     tie my %billToAddress, 'Tie::IxHash', $self->revmap_fields(
         name         => 'name',
@@ -461,13 +474,31 @@ sub map_request {
           );
     }
     elsif ( $action eq 'credit' ) {
-        push @required_fields, qw( order_number amount );
-        tie %req, 'Tie::IxHash', $self->revmap_fields(
-            litleTxnId    => 'order_number',
-            amount        => \$amount,
-            customBilling => \%custombilling,
-            processingInstructions  =>  \%processing,
-        );
+
+       # IF there is a litleTxnId, it's a normal linked credit
+       if( $content->{'order_number'} ){
+          push @required_fields, qw( order_number amount );
+          tie %req, 'Tie::IxHash', $self->revmap_fields(
+              litleTxnId    => 'order_number',
+              amount        => \$amount,
+              customBilling => \%custombilling,
+              processingInstructions  =>  \%processing,
+          );
+        }
+       # ELSE it's an unlinked, which requires different data
+       else {
+          push @required_fields, qw( invoice_number amount );
+          tie %req, 'Tie::IxHash', $self->revmap_fields(
+              orderId       => 'invoice_number',
+              amount        => \$amount,
+              orderSource   => 'orderSource',
+              billToAddress => \%billToAddress,
+              card          => \%card,
+              token         => $content->{'token'} ? \%token : {},
+              customBilling => \%custombilling,
+              processingInstructions  =>  \%processing,
+          );
+       }
     }
     elsif ( $action eq 'void' ) {
         push @required_fields, qw( order_number );
@@ -535,12 +566,19 @@ sub submit {
     );
 
     $self->_xmlwrite( $writer, 'authentication', \%authentication );
+
+    ## partial capture modifier, odd location, because it modifies the start tag :(
+    my %extra;
+    if ($content{'TransactionType'} eq 'capture'){
+        $extra{'partial'} = $content{'partial'} ? 'true' : 'false';
+    }
+
     $writer->startTag(
         $content{'TransactionType'},
         id          => $content{'invoice_number'},
         reportGroup => $content{'report_group'} || 'BOP',
         customerId  => $content{'customer_id'} || 1,
-        #partial     => $content{'partial'} ? 'true' : 'false',
+        %extra,
     );
     foreach ( keys( %{$req} ) ) {
         $self->_xmlwrite( $writer, $_, $req->{$_} );
@@ -563,7 +601,7 @@ sub submit {
         if ( exists( $response->{'response'} ) && $response->{'response'} == 1 )
         {
             ## parse error type error
-            print Dumper( $response, $self->{'_post_data'} );
+            warn Dumper( $response, $self->{'_post_data'} );
             $self->error_message( $response->{'message'} );
             return;
         }
@@ -600,6 +638,8 @@ sub submit {
       $self->is_prepaid(0);
     }
 
+    #$self->is_dupe( $resp->{'duplicate'} ? 1 : 0 );
+
     if( $resp->{enhancedAuthResponse}
         && $resp->{enhancedAuthResponse}->{affluence} 
       ){
@@ -774,7 +814,7 @@ sub create_batch {
                 && $response->{'response'} == 1 )
             {
                 ## parse error type error
-                print Dumper( $response, $self->{'_post_data'} );
+                warn Dumper( $response, $self->{'_post_data'} );
                 $self->error_message( $response->{'message'} );
                 return;
             }
@@ -876,7 +916,7 @@ sub send_rfr {
         if ( exists( $response->{'response'} ) && $response->{'response'} == 1 )
         {
             ## parse error type error
-            print Dumper( $response, $self->{'_post_data'} );
+            warn Dumper( $response, $self->{'_post_data'} );
             $self->error_message( $response->{'message'} );
             return;
         }
@@ -938,7 +978,7 @@ sub retrieve_batch {
     $response = XMLin($post_data);
     if ( exists( $response->{'response'} ) && $response->{'response'} == 1 ) {
         ## parse error type error
-        print Dumper( $response, $self->{'_post_data'} );
+        warn Dumper( $response, $self->{'_post_data'} );
         $self->error_message( $response->{'message'} );
         return;
     }
